//Written for www.HomoFaciens.de
//Program gives informations between different measuring units and shows the conversion coefficient.
//Copyright (C) 2009 Norbert Heinz
//
//This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free //Software Foundation version 3 of the License.
//
//This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or //FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License along with this program; if not, see http://www.gnu.org/licenses/.

import java.awt.*;
import java.awt.event.*;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import java.io.*;
import java.net.URL;
import javax.swing.*;
import java.text.*;



public class measurement extends JApplet implements  ActionListener  {

    static String versionNumber="v1.4";

    int sceneHeight=200, sceneWidth=900;
    Box dummyBoxCol1 = Box.createHorizontalBox(); //Empty boxes for creating fixed row width and column height
    Box dummyBoxCol3 = Box.createHorizontalBox();
    Box dummyBoxRow1 = Box.createHorizontalBox();
    Box dummyBoxRow2 = Box.createHorizontalBox();
    Box dummyBoxRow3 = Box.createHorizontalBox();
    String[] distanceUnitsEnglish={"Meters", "Kilometers", "Decimeters", "Centimeters", "Millimeters", "Angstrom", "Statute miles", "Nautical miles", "Inches", "Astronomical Units", "Lightyears", "Parsec", "yards", "feet"};
    String[] distanceUnitsGerman={"Meter", "Kilometer", "Dezimeter", "Zentimeter", "Millimeter", "Angstrom", "Britische Landmeilen", "Nautische Meilen", "Zoll", "Astronomische Einheiten", "Lichtjahre", "Parsec", "Yards", "Fuss"};
    double[] distanceMultiplier={1.0, 1000.0, 0.1, 0.01, 0.001, 0.0000000001, 1609.334, 1852.0, 0.0254, 149597870691.0, 9460730472580800.0, 30856776000000000.0, 0.9144, 0.3048};
    String[] speedUnitsEnglish={"Meter per second", "Kilometer per hour", "Miles per hour", "Knots", "Mach", "Lightspeed"};
    String[] speedUnitsGerman={"Meter pro Sekunde", "Kilometer pro Stunde", "Meilen pro Stunde", "Knoten", "Mach", "Lichtgeschwindigkeit"};
    double[] speedMultiplier={1.0, 3.6, 0.4470372222222222222, 0.51444, 343.0, 299792458.0};
    String[] weightUnitsEnglish={"Kilogram", "Gramme", "Pounds", "Tons", "Ounces", "Unified atomic mass units"};
    String[] weightUnitsGerman={"Kilogramm", "Gramm", "Pfund", "Tonnen", "Unzen", "Atomare Masseneinheiten"};
    double[] weightMultiplier={1.0, 0.001, 0.45359237, 1000.0, 0.028349523125, 1.660538782E-27};
    String[] timeUnitsEnglish={"Seconds", "Minutes", "Hours", "Days", "Weeks", "Months", "Years"};
    String[] timeUnitsGerman={"Sekunden", "Minuten", "Stunden", "Tage", "Wochen", "Monate", "Jahre"};
    double[] timeMultiplier={1.0, 60.0, 3600.0, 86400.0, 604800.0, 2551440.0, 31558149.5};
    String[] volumeUnitsEnglish={"Liter", "Cubic meter", "Cubic centimeter", "Microliter"};
    String[] volumeUnitsGerman={"Liter", "Kubikmeter", "Kubikzentimeter", "Mikroliter"};
    double[] volumeMultiplier={1.0, 1000.0, 0.001, 0.000001};
    String[] energyUnitsEnglish={"Joule", "Calories", "Kilowatt hours"};
    String[] energyUnitsGerman={"Joule", "Kalorien", "Kilowattstunden"};
    double[] energyMultiplier={1.0,  4.182, 3600000.0};
    String[] powerUnitsEnglish={"Watts", "Horsepower", "Kilowatts"};
    String[] powerUnitsGerman={"Watt", "Pferdestaerken", "Kilowatt"};
    double[] powerMultiplier={1.0, 735.49875 ,  1000.0};
    String[] temperatureUnitsEnglish={"Kelvin", "Celsius", "Farenheit"};
    String[] temperatureUnitsGerman={"Kelvin", "Celsius", "Farenheit"};
    double[] temperatureMultiplier={1.0, 1000.0, 1.0};
    String[] angleUnitsEnglish={"Radian", "Degrees", "Arcminutes", "Point"};
    String[] angleUnitsGerman={"Rad", "Grad", "Bogenminuten", "Strich"};
    double[] angleMultiplier={1.0/6.2831854, 1.0/360.0, 1.0/21600.0, 1.0/32.0};
    String[] areaUnitsEnglish={"Square meters", "Hectare", "Morgen", "Are", "Barn"};
    String[] areaUnitsGerman={"Quadratmeter", "Hektar", "Morgen", "Ar", "Barn"};
    double[] areaMultiplier={1.0, 10000.0, 2500.0, 100.0, 1.0E-28};
    JSpinner givenValueSpinner;
    SpinnerNumberModel givenValueNumberModel = new SpinnerNumberModel(1.0, 1.0, 100000000.0, 0.1);
    JRadioButton energyButton, speedButton, timeButton, distanceButton, weightButton, volumeButton, powerButton, temperatureButton, angleButton, areaButton;
    JComboBox sourceUnitComboBox, targetUnitComboBox;
    JLabel resultLabel, multiplierLabel;
    JLabel sourceLabel, versionLabel;
    static String language="initalValue";
    String givenParam="nothing yet";   //To applet passed arguments
    JEditorPane explanationPane01 = new JEditorPane();
    JEditorPane explanationPane02 = new JEditorPane();
    boolean stopCalculate=false;

    public void init() {
        UIManager.put("swing.boldMetal", Boolean.FALSE);

        try{
          givenParam = getParameter("LanguageSettings");
        }
        catch(NullPointerException npe){
          System.out.println("could not get parameter 'LanguageSettings'");
        }

        if(givenParam.compareTo("nothing yet")!=0) {
          if (givenParam.compareTo("German")==0)language="German";
          else language="English";
        }
        //System.out.println("language at init()=>" + language + "<"); // Display the string.
    }

    public void start() {
      int i;

        initComponents();
    }

    public static void main(String[] args) {

        if(args.length>0){
          if (args[0].compareTo("German")==0)language="German";
          else language="English";
        }
        else{
          language="English";
        }

        JFrame f = new JFrame("Measurement");
        f.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

        JApplet ap = new measurement();
        ap.init();
        ap.start();
        f.getContentPane().add("Center", ap);
        f.pack();
        f.setVisible(true);

    }

    public void initComponents() {
        int i;
        JFormattedTextField ftf = null;
        GridBagConstraints c = new GridBagConstraints();
        Font font = new Font("Arial", Font.PLAIN, 16);
        Font smallFont = new Font("Arial", Font.PLAIN, 12);

        Box dummyBoxCol1 = Box.createHorizontalBox(); //Empty boxes for creating fixed row width and column height
        Box dummyBoxCol2 = Box.createHorizontalBox();
        Box dummyBoxCol3 = Box.createHorizontalBox();
        Box dummyBoxCol4 = Box.createHorizontalBox();
        Box dummyBoxRow1 = Box.createHorizontalBox();
        Box dummyBoxRow2 = Box.createHorizontalBox();
        Box dummyBoxRow3 = Box.createHorizontalBox();

        Box dummyBoxCol1b = Box.createHorizontalBox(); //Empty boxes for creating fixed row width and column height
        Box dummyBoxCol2b = Box.createHorizontalBox();
        Box dummyBoxCol3b = Box.createHorizontalBox();
        Box dummyBoxCol4b = Box.createHorizontalBox();
        Box dummyBoxCol5b = Box.createHorizontalBox();

        getContentPane().setLayout(new BorderLayout());

        //create panels
        JPanel controlPanel = new JPanel();
        JPanel valuePanel = new JPanel();
        JPanel explanationPanel = new JPanel();
        JPanel mainPanel = new JPanel();

        //set panel properties
        controlPanel.setLayout(new GridBagLayout());
        valuePanel.setLayout(new GridBagLayout());
        explanationPanel.setLayout(new FlowLayout());
        mainPanel.setLayout(new BoxLayout(mainPanel, BoxLayout.Y_AXIS));

        givenValueSpinner = new JSpinner(givenValueNumberModel);
        ftf = getTextField(givenValueSpinner);
        if (ftf != null ) {
            ftf.setFont(font);
            ftf.setColumns(10); //specify more width than we need
            ftf.setHorizontalAlignment(JTextField.RIGHT);
        }
        givenValueSpinner.addChangeListener(listener);
        givenValueSpinner.setFont(font);

        if(language.compareTo("German")==0){
          energyButton = new JRadioButton("Energie");
          energyButton.setMnemonic(KeyEvent.VK_E);
          speedButton = new JRadioButton("Geschwindigkeit");
          speedButton.setMnemonic(KeyEvent.VK_G);
          timeButton = new JRadioButton("Zeit");
          timeButton.setMnemonic(KeyEvent.VK_Z);
          distanceButton = new JRadioButton("Länge");
          distanceButton.setMnemonic(KeyEvent.VK_L);
          weightButton = new JRadioButton("Masse");
          weightButton.setMnemonic(KeyEvent.VK_M);
          volumeButton = new JRadioButton("Volumen");
          volumeButton.setMnemonic(KeyEvent.VK_V);
          powerButton = new JRadioButton("Leistung");
          powerButton.setMnemonic(KeyEvent.VK_I);
          temperatureButton = new JRadioButton("Temperatur");
          temperatureButton.setMnemonic(KeyEvent.VK_T);
          angleButton = new JRadioButton("Winkel");
          angleButton.setMnemonic(KeyEvent.VK_W);
          areaButton = new JRadioButton("Fläche");
          areaButton.setMnemonic(KeyEvent.VK_F);
          sourceUnitComboBox = new JComboBox(distanceUnitsGerman);
          targetUnitComboBox = new JComboBox(distanceUnitsGerman);
        }
        else if(language.compareTo("XXX")==0){
          energyButton = new JRadioButton("XXX");
        }
        else{
          energyButton = new JRadioButton("Energy");
          energyButton.setMnemonic(KeyEvent.VK_E);
          speedButton = new JRadioButton("Speed");
          speedButton.setMnemonic(KeyEvent.VK_S);
          timeButton = new JRadioButton("Time");
          timeButton.setMnemonic(KeyEvent.VK_T);
          distanceButton = new JRadioButton("Length");
          distanceButton.setMnemonic(KeyEvent.VK_L);
          weightButton = new JRadioButton("Mass");
          weightButton.setMnemonic(KeyEvent.VK_M);
          volumeButton = new JRadioButton("Volume");
          volumeButton.setMnemonic(KeyEvent.VK_V);
          powerButton = new JRadioButton("Power");
          powerButton.setMnemonic(KeyEvent.VK_P);
          temperatureButton = new JRadioButton("Temperature");
          temperatureButton.setMnemonic(KeyEvent.VK_U);
          angleButton = new JRadioButton("Angle");
          angleButton.setMnemonic(KeyEvent.VK_A);
          areaButton = new JRadioButton("Area");
          areaButton.setMnemonic(KeyEvent.VK_R);
          sourceUnitComboBox = new JComboBox(distanceUnitsEnglish);
          targetUnitComboBox = new JComboBox(distanceUnitsEnglish);
        }

        sourceUnitComboBox.setFont(font);
        targetUnitComboBox.setFont(font);
        sourceUnitComboBox.setPreferredSize(new Dimension(190, 25));
        targetUnitComboBox.setPreferredSize(new Dimension(190, 25));

        ButtonGroup group = new ButtonGroup();
        group.add(speedButton);
        group.add(timeButton);
        group.add(distanceButton);
        group.add(volumeButton);
        group.add(weightButton);
        group.add(energyButton);
        group.add(powerButton);
        group.add(temperatureButton);
        group.add(angleButton);
        group.add(areaButton);

        distanceButton.setFont(font);
        distanceButton.setActionCommand("distanceButtonPressed");
        distanceButton.addActionListener(this);
        distanceButton.setSelected(true);

        speedButton.setFont(font);
        speedButton.setActionCommand("speedButtonPressed");
        speedButton.addActionListener(this);

        timeButton.setFont(font);
        timeButton.setActionCommand("timeButtonPressed");
        timeButton.addActionListener(this);

        energyButton.setFont(font);
        energyButton.setActionCommand("energyButtonPressed");
        energyButton.addActionListener(this);

        weightButton.setFont(font);
        weightButton.setActionCommand("weightButtonPressed");
        weightButton.addActionListener(this);

        volumeButton.setFont(font);
        volumeButton.setActionCommand("volumeButtonPressed");
        volumeButton.addActionListener(this);

        powerButton.setFont(font);
        powerButton.setActionCommand("powerButtonPressed");
        powerButton.addActionListener(this);

        temperatureButton.setFont(font);
        temperatureButton.setActionCommand("temperatureButtonPressed");
        temperatureButton.addActionListener(this);

        areaButton.setFont(font);
        areaButton.setActionCommand("areaButtonPressed");
        areaButton.addActionListener(this);

        angleButton.setFont(font);
        angleButton.setActionCommand("angleButtonPressed");
        angleButton.addActionListener(this);

        multiplierLabel = new JLabel("1.0");
        multiplierLabel.setFont(font);
        resultLabel = new JLabel("1.123");
        resultLabel.setFont(font);

        dummyBoxCol1.add(dummyBoxCol1.createRigidArea(new Dimension (170,1)));
        dummyBoxCol2.add(dummyBoxCol3.createRigidArea(new Dimension (170,1)));
        dummyBoxCol3.add(dummyBoxCol3.createRigidArea(new Dimension (170,1)));
        dummyBoxCol4.add(dummyBoxCol4.createRigidArea(new Dimension (170,1)));
        dummyBoxRow1.add(dummyBoxRow1.createRigidArea(new Dimension (1,20)));
        dummyBoxRow2.add(dummyBoxRow2.createRigidArea(new Dimension (1,20)));
        dummyBoxRow3.add(dummyBoxRow3.createRigidArea(new Dimension (1,20)));


        c.insets = new Insets(10,0,0,0);
        c.gridwidth=1;
        c.gridx = 0;
        c.gridy = 0;
        controlPanel.add(dummyBoxCol1, c);
        c.gridwidth=1;
        c.gridx = 1;
        c.gridy = 0;
        controlPanel.add(dummyBoxCol2, c);
        c.gridwidth=1;
        c.gridx = 2;
        c.gridy = 0;
        controlPanel.add(dummyBoxCol3, c);
        c.gridwidth=1;
        c.gridx = 3;
        c.gridy = 0;
        controlPanel.add(dummyBoxCol4, c);

        c.anchor=GridBagConstraints.LINE_START;
        c.gridwidth=1;
        c.gridx = 0;
        c.gridy = 1;
        controlPanel.add(distanceButton, c);
        c.gridx = 1;
        c.gridy = 1;
        controlPanel.add(speedButton, c);
        c.gridx = 2;
        c.gridy = 1;
        controlPanel.add(timeButton, c);
        c.gridx = 3;
        c.gridy = 1;
        controlPanel.add(powerButton, c);
        c.gridx = 4;
        c.gridy = 1;
        controlPanel.add(angleButton, c);

        c.gridx = 0;
        c.gridy = 2;
        controlPanel.add(energyButton, c);
        c.gridx = 1;
        c.gridy = 2;
        controlPanel.add(weightButton, c);
        c.gridx = 2;
        c.gridy = 2;
        controlPanel.add(volumeButton, c);
        c.gridx = 3;
        c.gridy = 2;
        controlPanel.add(temperatureButton, c);
        c.gridx = 4;
        c.gridy = 2;
        controlPanel.add(areaButton, c);

        dummyBoxCol1b.add(dummyBoxCol1.createRigidArea(new Dimension (170,1)));
        dummyBoxCol2b.add(dummyBoxCol1.createRigidArea(new Dimension (190,1)));
        dummyBoxCol3b.add(dummyBoxCol1.createRigidArea(new Dimension (120,1)));
        dummyBoxCol4b.add(dummyBoxCol1.createRigidArea(new Dimension (120,1)));
        dummyBoxCol5b.add(dummyBoxCol1.createRigidArea(new Dimension (190,1)));


        c.insets = new Insets(10,10,0,0);
        c.gridwidth=1;

        c.gridx = 0;
        c.gridy = 1;
        valuePanel.add(dummyBoxCol1b, c);
        c.gridx = 1;
        c.gridy = 1;
        valuePanel.add(dummyBoxCol2b, c);
        c.gridx = 2;
        c.gridy = 1;
        valuePanel.add(dummyBoxCol3b, c);
        c.gridx = 3;
        c.gridy = 1;
        valuePanel.add(dummyBoxCol4b, c);
        c.gridx = 4;
        c.gridy = 1;
        valuePanel.add(dummyBoxCol5b, c);

        c.gridx = 0;
        c.gridy = 1;
        valuePanel.add(givenValueSpinner, c);

        c.gridx = 1;
        c.gridy = 1;
        valuePanel.add(sourceUnitComboBox, c);

        c.anchor=GridBagConstraints.CENTER;
        c.gridx = 2;
        c.gridy = 1;
        valuePanel.add(multiplierLabel, c);

        c.anchor=GridBagConstraints.LINE_END;
        c.gridx = 3;
        c.gridy = 1;
        valuePanel.add(resultLabel,c);

        c.anchor=GridBagConstraints.LINE_START;
        c.gridx = 4;
        c.gridy = 1;
        valuePanel.add(targetUnitComboBox, c);

        sourceUnitComboBox.addActionListener(this);
        targetUnitComboBox.addActionListener(this);

        explanationPane01.setEditable(false);
        explanationPane02.setEditable(false);

        //Put the editor pane in a scroll pane.
        JScrollPane explanationScrollPane01 = new JScrollPane(explanationPane01);
        explanationScrollPane01.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
        explanationScrollPane01.setPreferredSize(new Dimension(420, 250));
        explanationScrollPane01.setMinimumSize(new Dimension(10, 10));

        JScrollPane explanationScrollPane02 = new JScrollPane(explanationPane02);
        explanationScrollPane02.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
        explanationScrollPane02.setPreferredSize(new Dimension(420, 250));
        explanationScrollPane02.setMinimumSize(new Dimension(10, 10));

        explanationPanel.add(explanationScrollPane01);
        explanationPanel.add(explanationScrollPane02);

        sourceUnitComboBox.setSelectedIndex(0);
        targetUnitComboBox.setSelectedIndex(0);

        if(language.compareTo("German")==0){
          setExplanation(explanationPane01, (String)sourceUnitComboBox.getSelectedItem() + "_ger");
          setExplanation(explanationPane02, (String)targetUnitComboBox.getSelectedItem() + "_ger");
        }
        else{
          setExplanation(explanationPane01, (String)sourceUnitComboBox.getSelectedItem() + "_en");
          setExplanation(explanationPane02, (String)targetUnitComboBox.getSelectedItem() + "_en");
        }

        sourceLabel = new JLabel(versionNumber + "         Source & info:    www.HomoFaciens.de               Comments based on www.wikipedia.org");
        sourceLabel.setFont(smallFont);

        mainPanel.add(controlPanel);
        mainPanel.add(valuePanel);
        mainPanel.add(explanationPanel);
        mainPanel.add(sourceLabel);

        sourceLabel.setAlignmentX(Component.CENTER_ALIGNMENT);

        getContentPane().add(mainPanel);

        calculate();

    }

    public void setExplanation(JEditorPane editPane, String fileName) {
        fileName=fileName.replaceAll(" ", "_");
        fileName=fileName.toLowerCase();
        fileName="text/" + fileName + ".html";
        java.net.URL fileURL = measurement.class.getResource(fileName);
        if (fileURL != null) {
            try {
                editPane.setPage(fileURL);
            } catch (IOException e) {
                editPane.setText("Attempted to read a bad URL: " + fileName);
                System.err.println("Attempted to read a bad URL: " + fileName);
            }
        } else {
            editPane.setText("Couldn't find file: " + fileName);
            System.err.println("Couldn't find file: " + fileName);
        }
    }

    public JFormattedTextField getTextField(JSpinner spinner) {
        JComponent editor = spinner.getEditor();
        if (editor instanceof JSpinner.DefaultEditor) {
            return ((JSpinner.DefaultEditor)editor).getTextField();
        } else {
            System.err.println("Unexpected editor type: "  + spinner.getEditor().getClass());
            return null;
        }
    }


    ChangeListener listener = new ChangeListener() {
      public void stateChanged(ChangeEvent e) {
        if(!stopCalculate)calculate();
      }
    };

    // Listens to the buttons.
    public void actionPerformed(ActionEvent e) {
        int i, i2;

        if(e.getActionCommand()=="speedButtonPressed"){
          stopCalculate=true;
          sourceUnitComboBox.removeAllItems();
          targetUnitComboBox.removeAllItems();
          if(language.compareTo("German")==0){
            for(i=0;i<speedUnitsGerman.length;i++){
              sourceUnitComboBox.addItem(speedUnitsGerman[i]);
              targetUnitComboBox.addItem(speedUnitsGerman[i]);
            }
          }
          else if(language.compareTo("XXX")==0){
            System.out.println("No valid language: " + language); // Display the string.
          }
          else{
            for(i=0;i<speedUnitsEnglish.length;i++){
              sourceUnitComboBox.addItem(speedUnitsEnglish[i]);
              targetUnitComboBox.addItem(speedUnitsEnglish[i]);
            }
          }
          stopCalculate=false;
        }
        if(e.getActionCommand()=="distanceButtonPressed"){
          stopCalculate=true;
          sourceUnitComboBox.removeAllItems();
          targetUnitComboBox.removeAllItems();
          if(language.compareTo("German")==0){
            for(i=0;i<distanceUnitsGerman.length;i++){
              sourceUnitComboBox.addItem(distanceUnitsGerman[i]);
              targetUnitComboBox.addItem(distanceUnitsGerman[i]);
            }
          }
          else if(language.compareTo("XXX")==0){
            System.out.println("No valid language: " + language); // Display the string.
          }
          else{
            for(i=0;i<distanceUnitsEnglish.length;i++){
              sourceUnitComboBox.addItem(distanceUnitsEnglish[i]);
              targetUnitComboBox.addItem(distanceUnitsEnglish[i]);
            }
          }
          stopCalculate=false;
        }
        if(e.getActionCommand()=="timeButtonPressed"){
          stopCalculate=true;
          sourceUnitComboBox.removeAllItems();
          targetUnitComboBox.removeAllItems();
          if(language.compareTo("German")==0){
            for(i=0;i<timeUnitsGerman.length;i++){
              sourceUnitComboBox.addItem(timeUnitsGerman[i]);
              targetUnitComboBox.addItem(timeUnitsGerman[i]);
            }
          }
          else if(language.compareTo("XXX")==0){
            System.out.println("No valid language: " + language); // Display the string.
          }
          else{
            for(i=0;i<timeUnitsEnglish.length;i++){
              sourceUnitComboBox.addItem(timeUnitsEnglish[i]);
              targetUnitComboBox.addItem(timeUnitsEnglish[i]);
            }
          }
          stopCalculate=false;
        }
        if(e.getActionCommand()=="energyButtonPressed"){
          stopCalculate=true;
          sourceUnitComboBox.removeAllItems();
          targetUnitComboBox.removeAllItems();
          if(language.compareTo("German")==0){
            for(i=0;i<energyUnitsGerman.length;i++){
              sourceUnitComboBox.addItem(energyUnitsGerman[i]);
              targetUnitComboBox.addItem(energyUnitsGerman[i]);
            }
          }
          else if(language.compareTo("XXX")==0){
            System.out.println("No valid language: " + language); // Display the string.
          }
          else{
            for(i=0;i<energyUnitsEnglish.length;i++){
              sourceUnitComboBox.addItem(energyUnitsEnglish[i]);
              targetUnitComboBox.addItem(energyUnitsEnglish[i]);
            }
          }
          stopCalculate=false;
        }
        if(e.getActionCommand()=="weightButtonPressed"){
          stopCalculate=true;
          sourceUnitComboBox.removeAllItems();
          targetUnitComboBox.removeAllItems();
          if(language.compareTo("German")==0){
            for(i=0;i<weightUnitsGerman.length;i++){
              sourceUnitComboBox.addItem(weightUnitsGerman[i]);
              targetUnitComboBox.addItem(weightUnitsGerman[i]);
            }
          }
          else if(language.compareTo("XXX")==0){
            System.out.println("No valid language: " + language); // Display the string.
          }
          else{
            for(i=0;i<weightUnitsEnglish.length;i++){
              sourceUnitComboBox.addItem(weightUnitsEnglish[i]);
              targetUnitComboBox.addItem(weightUnitsEnglish[i]);
            }
          }
          stopCalculate=false;
        }
        if(e.getActionCommand()=="volumeButtonPressed"){
          stopCalculate=true;
          sourceUnitComboBox.removeAllItems();
          targetUnitComboBox.removeAllItems();
          if(language.compareTo("German")==0){
            for(i=0;i<volumeUnitsGerman.length;i++){
              sourceUnitComboBox.addItem(volumeUnitsGerman[i]);
              targetUnitComboBox.addItem(volumeUnitsGerman[i]);
            }
          }
          else if(language.compareTo("XXX")==0){
            System.out.println("No valid language: " + language); // Display the string.
          }
          else{
            for(i=0;i<volumeUnitsEnglish.length;i++){
              sourceUnitComboBox.addItem(volumeUnitsEnglish[i]);
              targetUnitComboBox.addItem(volumeUnitsEnglish[i]);
            }
          }
          stopCalculate=false;
        }
        if(e.getActionCommand()=="powerButtonPressed"){
          stopCalculate=true;
          sourceUnitComboBox.removeAllItems();
          targetUnitComboBox.removeAllItems();
          if(language.compareTo("German")==0){
            for(i=0;i<powerUnitsGerman.length;i++){
              sourceUnitComboBox.addItem(powerUnitsGerman[i]);
              targetUnitComboBox.addItem(powerUnitsGerman[i]);
            }
          }
          else if(language.compareTo("XXX")==0){
            System.out.println("No valid language: " + language); // Display the string.
          }
          else{
            for(i=0;i<powerUnitsEnglish.length;i++){
              sourceUnitComboBox.addItem(powerUnitsEnglish[i]);
              targetUnitComboBox.addItem(powerUnitsEnglish[i]);
            }
          }
          stopCalculate=false;
        }
        if(e.getActionCommand()=="temperatureButtonPressed"){
          stopCalculate=true;
          sourceUnitComboBox.removeAllItems();
          targetUnitComboBox.removeAllItems();
          if(language.compareTo("German")==0){
            for(i=0;i<temperatureUnitsGerman.length;i++){
              sourceUnitComboBox.addItem(temperatureUnitsGerman[i]);
              targetUnitComboBox.addItem(temperatureUnitsGerman[i]);
            }
          }
          else if(language.compareTo("XXX")==0){
            System.out.println("No valid language: " + language); // Display the string.
          }
          else{
            for(i=0;i<temperatureUnitsEnglish.length;i++){
              sourceUnitComboBox.addItem(temperatureUnitsEnglish[i]);
              targetUnitComboBox.addItem(temperatureUnitsEnglish[i]);
            }
          }
          stopCalculate=false;
        }
        if(e.getActionCommand()=="angleButtonPressed"){
          stopCalculate=true;
          sourceUnitComboBox.removeAllItems();
          targetUnitComboBox.removeAllItems();
          if(language.compareTo("German")==0){
            for(i=0;i<angleUnitsGerman.length;i++){
              sourceUnitComboBox.addItem(angleUnitsGerman[i]);
              targetUnitComboBox.addItem(angleUnitsGerman[i]);
            }
          }
          else if(language.compareTo("XXX")==0){
            System.out.println("No valid language: " + language); // Display the string.
          }
          else{
            for(i=0;i<angleUnitsEnglish.length;i++){
              sourceUnitComboBox.addItem(angleUnitsEnglish[i]);
              targetUnitComboBox.addItem(angleUnitsEnglish[i]);
            }
          }
          stopCalculate=false;
        }
        if(e.getActionCommand()=="areaButtonPressed"){
          stopCalculate=true;
          sourceUnitComboBox.removeAllItems();
          targetUnitComboBox.removeAllItems();
          if(language.compareTo("German")==0){
            for(i=0;i<areaUnitsGerman.length;i++){
              sourceUnitComboBox.addItem(areaUnitsGerman[i]);
              targetUnitComboBox.addItem(areaUnitsGerman[i]);
            }
          }
          else if(language.compareTo("XXX")==0){
            System.out.println("No valid language: " + language); // Display the string.
          }
          else{
            for(i=0;i<areaUnitsEnglish.length;i++){
              sourceUnitComboBox.addItem(areaUnitsEnglish[i]);
              targetUnitComboBox.addItem(areaUnitsEnglish[i]);
            }
          }
          stopCalculate=false;
        }

        if(!stopCalculate){
          if(language.compareTo("German")==0){
            setExplanation(explanationPane01, (String)sourceUnitComboBox.getSelectedItem() + "_ger");
            setExplanation(explanationPane02, (String)targetUnitComboBox.getSelectedItem() + "_ger");
          }
          else{
            setExplanation(explanationPane01, (String)sourceUnitComboBox.getSelectedItem() + "_en");
            setExplanation(explanationPane02, (String)targetUnitComboBox.getSelectedItem() + "_en");
          }
          calculate();
        }
    }

    public void calculate() {
        double sourceValue, targetValue;
        double multiplier=1.0;

        sourceValue=givenValueNumberModel.getNumber().doubleValue();

        if(temperatureButton.isSelected()){//Temperature conversion is different
          multiplier=1.0;
          if(sourceUnitComboBox.getSelectedIndex()==0){
            if(targetUnitComboBox.getSelectedIndex()==0){//Kelvin to Kelvin
              multiplierLabel.setText(" ");
              resultLabel.setText(" = " +  new DecimalFormat("0.00000E00").format(multiplier * sourceValue));
            }
            if(targetUnitComboBox.getSelectedIndex()==1){//Kelvin to Celsius
              multiplierLabel.setText(new DecimalFormat("0.00000E00").format(sourceValue) + " - 273,15");
              resultLabel.setText(" = " +  new DecimalFormat("0.00000E00").format(sourceValue - 273.15));
            }
            if(targetUnitComboBox.getSelectedIndex()==2){//Kelvin to Fahrenheit
              multiplierLabel.setText(new DecimalFormat("0.00000E00").format(sourceValue) + " * 1,8 - 459,67");
              resultLabel.setText(" = " +  new DecimalFormat("0.00000E00").format(sourceValue * 1.8 - 459.67));
            }
          }
          if(sourceUnitComboBox.getSelectedIndex()==1){
            if(targetUnitComboBox.getSelectedIndex()==0){//Celsius to Kelvin
              multiplierLabel.setText(new DecimalFormat("0.00000E00").format(sourceValue) + " + 273,15");
              resultLabel.setText(" = " +  new DecimalFormat("0.00000E00").format(sourceValue + 273.15));
            }
            if(targetUnitComboBox.getSelectedIndex()==1){//Celsius to Celsius
              multiplierLabel.setText(" ");
              resultLabel.setText(" = " +  new DecimalFormat("0.00000E00").format(multiplier * sourceValue));
            }
            if(targetUnitComboBox.getSelectedIndex()==2){//Celsius to Fahrenheit
              multiplierLabel.setText(new DecimalFormat("0.00000E00").format(sourceValue) + " * 1,8 + 32.0");
              resultLabel.setText(" = " +  new DecimalFormat("0.00000E00").format(sourceValue * 1.8 + 32.0));
            }
          }
          if(sourceUnitComboBox.getSelectedIndex()==2){
            if(targetUnitComboBox.getSelectedIndex()==0){//Fahrenheit to Kelvin
              multiplierLabel.setText("( " + new DecimalFormat("0.00000E00").format(sourceValue) + " + 459,67 ) / 1,8");
              resultLabel.setText(" = " +  new DecimalFormat("0.00000E00").format((sourceValue + 459.67)/1.8));
            }
            if(targetUnitComboBox.getSelectedIndex()==1){//Fahrenheit to Celsius
              multiplierLabel.setText(new DecimalFormat("0.00000E00").format(sourceValue) + " * 1,8 + 32.0");
              resultLabel.setText(" = " +  new DecimalFormat("0.00000E00").format(sourceValue * 1.8 + 32.0));
            }
            if(targetUnitComboBox.getSelectedIndex()==2){//Fahrenheit to Fahrenheit
              multiplierLabel.setText(" ");
              resultLabel.setText(" = " +  new DecimalFormat("0.00000E00").format(multiplier * sourceValue));
            }
          }
        }
        else{//No temperature conversion, simple multiplications
          if(distanceButton.isSelected()){
            multiplier=distanceMultiplier[sourceUnitComboBox.getSelectedIndex()];
            multiplier=multiplier/distanceMultiplier[targetUnitComboBox.getSelectedIndex()];
          }
          if(speedButton.isSelected()){
            multiplier=speedMultiplier[sourceUnitComboBox.getSelectedIndex()];
            multiplier=multiplier/speedMultiplier[targetUnitComboBox.getSelectedIndex()];
          }
          if(timeButton.isSelected()){
            multiplier=timeMultiplier[sourceUnitComboBox.getSelectedIndex()];
            multiplier=multiplier/timeMultiplier[targetUnitComboBox.getSelectedIndex()];
          }
          if(weightButton.isSelected()){
            multiplier=weightMultiplier[sourceUnitComboBox.getSelectedIndex()];
            multiplier=multiplier/weightMultiplier[targetUnitComboBox.getSelectedIndex()];
          }
          if(energyButton.isSelected()){
            multiplier=energyMultiplier[sourceUnitComboBox.getSelectedIndex()];
            multiplier=multiplier/energyMultiplier[targetUnitComboBox.getSelectedIndex()];
          }
          if(powerButton.isSelected()){
            multiplier=powerMultiplier[sourceUnitComboBox.getSelectedIndex()];
            multiplier=multiplier/powerMultiplier[targetUnitComboBox.getSelectedIndex()];
          }
          if(volumeButton.isSelected()){
            multiplier=volumeMultiplier[sourceUnitComboBox.getSelectedIndex()];
            multiplier=multiplier/volumeMultiplier[targetUnitComboBox.getSelectedIndex()];
          }
          if(areaButton.isSelected()){
            multiplier=areaMultiplier[sourceUnitComboBox.getSelectedIndex()];
            multiplier=multiplier/areaMultiplier[targetUnitComboBox.getSelectedIndex()];
          }
          if(angleButton.isSelected()){
            multiplier=angleMultiplier[sourceUnitComboBox.getSelectedIndex()];
            multiplier=multiplier/angleMultiplier[targetUnitComboBox.getSelectedIndex()];
          }
          multiplierLabel.setText(" * " + new DecimalFormat("0.00000E00").format(multiplier));
          resultLabel.setText(" = " +  new DecimalFormat("0.00000E00").format(multiplier * sourceValue));
       }

    }

}



